@@ -2,7 +2,7 @@ package Mojolicious::Plugin::Util::RandomString;
 use Mojo::Base 'Mojolicious::Plugin';
 use Session::Token;
 
-our $VERSION = '0.03_6';
+our $VERSION = '0.08_1';
 
 our (%generator, %setting, %default, %param);
 our $read_config;
@@ -23,6 +23,7 @@ sub register {
     $param{$_} = $param->{$_} foreach keys %$param;
   };
 
+
   # Load parameter from Config file
   unless ($read_config) {
     if (my $config_param = $mojo->config('Util-RandomString')) {
@@ -40,28 +41,28 @@ sub register {
       # Create generators by param
       foreach (keys %param) {
 
-	# Named generator
-	if (ref $param{$_} && ref $param{$_} eq 'HASH') {
+        # Named generator
+        if (ref $param{$_} && ref $param{$_} eq 'HASH') {
 
-	  next if $created{$_};
+          next if $created{$_};
 
-	  # Construct object
-	  unless ($generator{$_} = Session::Token->new(
-	    %{ $param{$_} }
-	  )) {
+          # Construct object
+          unless ($generator{$_} = Session::Token->new(
+            %{ $param{$_} }
+          )) {
 
-	    # Unable to construct object
-	    $mojo->log->fatal(qq!Unable to create generator for "$_"!);
-	    next;
-	  };
-	  $setting{$_} = { %{$param{$_}} };
-	  $created{$_} = 1;
-	}
+            # Unable to construct object
+            $mojo->log->fatal(qq!Unable to create generator for "$_"!);
+            next;
+          };
+          $setting{$_} = { %{$param{$_}} };
+          $created{$_} = 1;
+        }
 
-	# Default parameter
-	else {
-	  $default{$_} = $param{$_};
-	};
+        # Default parameter
+        else {
+          $default{$_} = $param{$_};
+        };
       };
 
       # Plugin registered
@@ -69,19 +70,11 @@ sub register {
 
       # Create default generator
       unless (exists $generator{default}) {
-	$generator{default} = Session::Token->new( %default );
-	warn 'Init S::T with {' . join(', ', map { $_ . ' => ' . (ref $default{$_} ? '[' . join(',', @{$default{$_}}) . ']' : $default{$_})} keys %default) . '} (' . $$ . ' ' . $plugin . ')';
+        $generator{default} = Session::Token->new( %default );
       };
     });
 
 
-  # Temporary
-  $mojo->helper(
-    _dump_random_string => sub {
-      return $$ . ' ' . $plugin . '{' . join(', ', map { $_ . ' => ' . (ref $default{$_} ? '[' . join(',', @{$default{$_}}) . ']' : $default{$_})} keys %default) . '}'
-    }
-  );
-
   # Establish 'random_string' helper
   $mojo->helper(
     random_string => sub {
@@ -89,43 +82,45 @@ sub register {
       my $gen = $_[0];
 
       # One tick for loop until the plugin is registered
-      until (Mojo::IOLoop->is_running || ($ok > 0)) {
-	warn 'TICK';
-	Mojo::IOLoop->one_tick;
-      };
-
+      Mojo::IOLoop->one_tick until Mojo::IOLoop->is_running || $ok > 0;
 
       # Generate from generator
       unless ($_[1]) {
 
-	# Generator doesn't exist
-	if ($gen && !exists $generator{$gen}) {
-	  $mojo->log->warn(qq!RandomString generator "$gen" is unknown!);
-	  return '';
-	};
+        # Generator doesn't exist
+        if ($gen && !exists $generator{$gen}) {
+          $c->app->log->warn(qq!RandomString generator "$gen" is unknown!);
+          return '';
+        };
 
-	warn 'GEN WAS ' . ($gen ? 'SET' : 'DEFAULT') . ': ' . $c->_dump_random_string . '(Example: ' . $generator{'default'}->get . ')';
-
-	# Get from generator
-	return $generator{$gen || 'default'}->get;
+        # Get from generator
+        return $generator{$gen || 'default'}->get;
       };
 
-      # Controller
-      # shift;
+      $gen = @_ % 2 ? shift : 'default';
 
-      # Overwrite default configuration
-      return Session::Token->new(%default, @_)->get unless @_ % 2;
+      # Overwrite configuration
+      my %overwrite = @_;
 
-      $gen = shift;
-      return Session::Token->new(%default, @_)->get if $gen eq 'default';
+      if (exists $overwrite{entropy}) {
+        $overwrite{length} //= undef;
+      }
+      elsif (exists $overwrite{length}) {
+        $overwrite{entropy} //= undef;
+      };
+
+      # Take default configuration
+      if ($gen eq 'default') {
+        return Session::Token->new(%default, %overwrite)->get;
+      }
 
       # Overwrite specific configuration
-      if ($setting{ $gen }) {
-	return Session::Token->new( %{ $setting{ $gen } } , @_)->get;
+      elsif ($setting{ $gen }) {
+        return Session::Token->new( %{ $setting{ $gen } }, %overwrite)->get;
       };
 
       # Generator is unknown
-      $mojo->log->warn(qq!RandomString generator "$gen" is unknown!);
+      $c->app->log->warn(qq!RandomString generator "$gen" is unknown!);
       return '';
     }
   ) unless exists $mojo->renderer->helpers->{random_string};
@@ -220,7 +215,7 @@ Expects a hash reference containing parameters as defined in
 L<Session::Token> for the default generator.
 To specify named generators, use a name key (other than C<alphabet>,
 C<length>, and C<entropy>) and specify the parameters as a hash reference.
-The name key 'default' can overwrite the default configuration.
+The name key C<default> can overwrite the default configuration.
 
 All parameters can be set either on registration or
 as part of the configuration file with the key C<Util-RandomString>.
@@ -272,9 +267,9 @@ L<Session::Token>.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2013-2014, L<Nils Diewald|http://nils-diewald.de/>.
+Copyright (C) 2013-2021, L<Nils Diewald|http://nils-diewald.de/>.
 
 This program is free software, you can redistribute it
-and/or modify it under the same terms as Perl.
+and/or modify it under the terms of the Artistic License version 2.0.
 
 =cut
